#ifndef __gtk_hst1d_h__
#define __gtk_hst1d_h__

#include <gtk/gtkimage.h>
#include <gtk-hst/function.h>

G_BEGIN_DECLS
                                                                                
#define GTK_TYPE_HST1D                  (gtk_hst1d_get_type ())
#define GTK_HST1D(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_HST1D, GtkHst1D))
#define GTK_HST1D_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_HST1D, GtkHst1DClass))
#define GTK_IS_HST1D(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_HST1D))
#define GTK_IS_HST1D_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_HST1D))
#define GTK_HST1D_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_HST1D, GtkHst1DClass))

typedef enum {
    HST_COLOR_FOREGROUND,
    HST_COLOR_BACKGROUND,
    HST_COLOR_AXIS,
    HST_NCOLORS
}  GtkHstColors;
GType gtk_hst_colors_get_type (void);
#define GTK_TYPE_HST_COLORS (gtk_hst_colors_get_type())

typedef enum {
   HST_AXIS_NONE   = 0,
   HST_AXIS_LEFT   = 1,
   HST_AXIS_MIDDLE = 2,
   HST_AXIS_RIGHT  = 4,
   HST_AXIS_X_MASK = 0x0f,
   HST_AXIS_Y_MASK = 0xf0,
   HST_AXIS_Z_MASK = 0xf00
} GtkHstAxisPos;
GType gtk_hst_axis_pos_get_type (void);
#define GTK_TYPE_HST_AXIS_POS (gtk_hst_axis_pos_get_type())


typedef enum {
   HST_FILLED = 1,
   HST_LOGX   = 2,
   HST_LOGY   = 4,
   HST_STATS  = 8
} GtkHstAttr;
GType gtk_hst_attr_get_type(void);
#define GTK_TYPE_HST_ATTR ( gtk_hst_attr_get_type() )

typedef enum {
    HST_X_AXIS,
    HST_Y_AXIS,
    HST_Z_AXIS
} GtkHstAxisType;
GType gtk_hst_axis_type(void);
#define GTK_TYPE_HST_AXIS_TYPE ( gtk_hst_axis_type() )


typedef struct _GtkHst1D       GtkHst1D;
typedef struct _GtkHst1DClass  GtkHst1DClass;
typedef struct _Canvas             Canvas;
typedef struct _GtkHst1DEvent  GtkHst1DEvent;
struct _GtkHst1D {
    GtkWidget obj;
    Canvas        *cnvs;
    HstFunction   *func;
    GdkPixmap     *pm;
    GdkGC         *gc;
    GdkColor       colors[HST_NCOLORS]; /* =0 foreground =1 background =2 axis */
    GList         *curves;
    GtkHst1DEvent *evt;
    Tuple         *xpt;
    char          *xtitle,*ytitle;
    
    
    int           tmout;
    int           sync_id;
    unsigned long drawaxis;
    double        xmin,xmax;
    double        ymin,ymax;
    double        vmin,vmax;
    gboolean      colors_changed;
    guint         attr;
};

struct _GtkHst1DClass {
    GtkWidgetClass parent_class;
      
    void (*draw_pixmap) (GtkHst1D *h);
    void (*draw_frame) (GtkHst1D *h);
    void (*draw_stats) (GtkHst1D *h);
    void (*draw_axis) (GtkHst1D *h);
    void (* mouse ) (GtkHst1D *h);
    void (*xtra_menu)(GtkHst1D *h, GtkWidget *w);
};

GType      gtk_hst1d_get_type (void) G_GNUC_CONST;
GtkWidget *gtk_hst1d_new(HstFunction *ff);
void       gtk_hst1d_add_curve(GtkHst1D *h, HstFunction *f);
void       gtk_hst1d_remove_curves(GtkHst1D *h);
void       gtk_hst1d_monitor_start (GtkHst1D *h, int tim);
void       gtk_hst1d_monitor_stop (GtkHst1D *h);
void       gtk_hst1d_draw_x_axis (GtkHst1D *h, GtkHstAxisPos val);
void       gtk_hst1d_draw_y_axis (GtkHst1D *h, GtkHstAxisPos val);
gchar      *gtk_hst1d_get_xtitle (GtkHst1D *hst);
void       gtk_hst1d_set_xtitle (GtkHst1D *hst, const char *tit);
gchar      *gtk_hst1d_get_ytitle (GtkHst1D *hst);
void       gtk_hst1d_set_ytitle (GtkHst1D *hst, const char *tit);
void       gtk_hst1d_filled (GtkHst1D *h, gboolean z);
gboolean   gtk_hst1d_is_filled (GtkHst1D *h);
void       gtk_hst1d_logx (GtkHst1D *h, gboolean z);
gboolean   gtk_hst1d_is_logx (GtkHst1D *h);
void       gtk_hst1d_logy (GtkHst1D *h, gboolean z);
gboolean   gtk_hst1d_is_logy (GtkHst1D *h);
void       gtk_hst1d_show_stats(GtkHst1D *h, gboolean z);
gboolean   gtk_hst1d_stats(GtkHst1D *h);

void         gtk_hst1d_set_curve_colors(GtkHst1D *h,...);
void         gtk_hst1d_add_curve_color(GtkHst1D *h, gchar *color_name);
int          gtk_hst1d_get_n_func(GtkHst1D *h);
HstFunction *gtk_hst1d_get_func(GtkHst1D *h, int n);
void         gtk_hst1d_set_func(GtkHst1D *h, HstFunction *f);
void         gtk_hst1d_sync_range(GtkHst1D *h, gboolean z);
/*
 * Sets the colors for:
 *   - foreground
 *   - background
 *   - axis
 * The values should be any accepted by gdk_color_parser
 * The last argument should be NULL
 *
 *   gtk_hst1d_set_colors(h,"foreground","black","axis","red",0)
 */
void       gtk_hst1d_set_colors(GtkHst1D *h,...);
G_END_DECLS
#endif
