#include <stdio.h>
#include <math.h>
#include <gtk/gtk.h>

#include <gtk-hst/gtkhistogram2d.h>
#include <gtk-hst/histogram2d.h>
#include <gtk-hst/histogram.h>
#include <gtk-hst/gtkhistogram.h>

#include "random.h"

GtkWidget *create_mainWindow (void);

GPrintFunc std_logger;
void my_logger(const gchar *string) {
    printf("[mylogger]%s\n",string);
}

void 
get_gaus2d(double m1, double s1, double m2, double s2, double corr, double *x, double *y)
{
    double f1,f2;
    double v1,v2;
    f1 = sqrt((1.+corr)/2.);
    f2 = sqrt((1.-corr)/2.);

    v1 = randgauss( 0.,1.);
    v2 = randgauss( 0.,1.);

    *x = m1 + s1*(f1*v1+f2*v2);
    *y = m2 + s2*(f1*v1-f2*v2);
}


gboolean
idle_func (GtkWidget *mainW)
{
    gulong ms;
    double x, y, r=randuniform();
    GObject *O = G_OBJECT(mainW);
    
    if (!O)
        return FALSE;
    
    HstHistogram2D *h2d= HST_HISTOGRAM2D( g_object_get_data(O,"h2d" ));

    if (h2d)
    {
        get_gaus2d(-5., 2., -5., 2., 0.5, &x, &y);
        hst_histogram2d_fill(h2d, x, y, 1.);

        get_gaus2d( 5., 2., 5., 2., 0.5, &x, &y);
        hst_histogram2d_fill(h2d, x, y, 1.);

        get_gaus2d(-5., 2., 5., 2., -0.5, &x, &y);
        hst_histogram2d_fill(h2d, x, y, -1.);
        get_gaus2d(5., 2., -5., 2., -0.5, &x, &y);
        hst_histogram2d_fill(h2d, x, y, -1.);
    }
    
    HstHistogram *h1d= HST_HISTOGRAM( g_object_get_data(O,"h1d" ));
    if (h1d)
    {
        hst_histogram_fill(h1d, randgauss( 0.,4.), 1);
    }
    
    return TRUE;
}


static gboolean 
delete_event( GtkWidget *widget,
              GdkEvent  *event,
              gpointer   data )
{
    /* Do not destroy with a delete-event */
    return TRUE;
}

int main (int argc, char *argv[]) {
  GtkWidget *mainWindow;
  int iid;

  //std_logger = g_set_printerr_handler( my_logger );
  
  gtk_init (&argc, &argv);
  
  mainWindow = create_mainWindow ();

  iid = g_idle_add((GSourceFunc)idle_func, mainWindow);
  printf ("idle id: %d\n", iid);
  gtk_dialog_run(GTK_DIALOG(mainWindow));
  return 0;
}

GtkWidget *
create_mainWindow (void)
{
    GtkWidget *mainWindow;
    GtkWidget *hst2d, *hst1d;
    HstHistogram2D *h2d;
    
    mainWindow = gtk_dialog_new_with_buttons(
                         "H2D", NULL,
					     GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
					     GTK_STOCK_OK,
					     GTK_RESPONSE_ACCEPT,
					     NULL
	);

    h2d   = hst_histogram2d_new (101,-25.,25.,101,-25.,25.,0);
    hst2d = gtk_hst2d_new ( HST_HISTOGRAM2D(h2d) );
    g_object_set_data( G_OBJECT(mainWindow),"h2d",h2d);
    gtk_hst1d_monitor_start(GTK_HST1D(hst2d),500);
    
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(mainWindow)->vbox), hst2d);

    gtk_widget_show_all(mainWindow);
    return mainWindow;
}
