#include <stdio.h>
#include <math.h>
#include <gtk/gtk.h>
#include <gtk-hst/gtkhistogram.h>
#include <gtk-hst/tracer.h>
#include <gtk-hst/gtkhistogram2d.h>
#include <gtk-hst/histogram2d.h>
#include <gtk-hst/gtkgauge.h>
#include <gtk-hst/graph.h>
#include <gtk-hst/profile.h>
#include "random.h"

GtkWidget *create_mainWindow(void);

GPrintFunc std_logger;
void my_logger(const gchar *string)
{
    std_logger(string);
}

typedef struct _SFunc SFunc;
struct _SFunc
{
    double freq;
    double mean;
    double sigma;
    HstFunction *ff;
};
GList *LFunc;
HstTracer *tracer=0;

double tanh_plus_one(double x)
{
    return tanh(x-2.);
}

double erf_plus_one(double x)
{
    return erf(x+2);
}

/* This pointer contains the GtkHst1D that we will use for testing the
 * range synchrnization.
 */
GtkHst1D *theHst=0;

void range_changed(HstFunction *f)
{
    g_print("range_changed\n");
}

gint quitting_gtk(gpointer ptr)
{
    printf("Adios !!!\n");
    gtk_widget_destroy( GTK_WIDGET(ptr));
    return 0;
}

void get_gaus2d(double m1, double s1, double m2, double s2, double corr, double *x, double *y)
{
    double f1, f2;
    double v1, v2;
    f1 = sqrt((1.+corr)/2.);
    f2 = sqrt((1.-corr)/2.);

    v1 = randgauss( 0., 1.);
    v2 = randgauss( 0., 1.);

    *x = m1 + s1*(f1*v1+f2*v2);
    *y = m2 + s2*(f1*v1-f2*v2);
}

GTimer *range_timer= NULL;
gint idle_func(GtkWidget *mainW)
{
    gulong ms;
    static double xmin0=-1;
    static double xmin1=-50.;

    HstProfile *hprof= HST_PROFILE( g_object_get_data(G_OBJECT(mainW),"hprof" ));
    HstHistogram2D *h2d= HST_HISTOGRAM2D( g_object_get_data(G_OBJECT(mainW),"h2d" ));
    GtkGauge *gg= GTK_GAUGE(g_object_get_data(G_OBJECT(mainW),"gauge" ));
    static gdouble p2 = 2.*M_PI, tim=0, dtim= 2.*M_PI/100.;
    gdouble x, r = randuniform();
    GList *lf;

    if (h2d)
    {
        double x, y;
        get_gaus2d(-5., 2., -5., 2., 0.5, &x, &y);
        hst_histogram2d_fill(h2d, x, y, 1.);
        get_gaus2d(5., 2., 5., 2., -0.5, &x, &y);
        hst_histogram2d_fill(h2d, x, y, 0.33);
    }

    if (tracer )
    {
        tim += dtim;
        if (tim > p2 )
            tim = 0;
        hst_histogram_fill((HstHistogram *)tracer, sin(tim), 1.);
    }

    if (hprof)
    {
        int it;
        int np = hst_function_npts( HST_FUNCTION(hprof));
        int nt = (int)(np*randuniform());
        for (it=0; it<nt; it++)
        {
            // select 1 channel
            int ib = (int)(np*randuniform() );
            double x = hst_function_bin_center( HST_FUNCTION(hprof), ib);
            hst_histogram_fill(HST_HISTOGRAM(hprof), x, randgauss(-(x-5.)*(x-5.)+10., 2.));
        }
    }
    lf = g_list_first(LFunc);
    for (; lf;)
    {
        SFunc *sf = (SFunc *)lf->data;
        if (sf->freq>r)
        {
            x = randgauss(sf->mean, sf->sigma);
            hst_histogram_fill((HstHistogram *)sf->ff, x, 1.);
        }
        lf = g_list_next (lf);
    }

    if (g_timer_elapsed(range_timer, &ms)>5.&& theHst)
    {
        double xmin;
        HstFunction *f;
        g_timer_reset(range_timer);
        g_print("Changing range to get the histogram synchronized\n");
        g_object_get(G_OBJECT(theHst),"func",&f,NULL);
        if (f)
        {
            g_object_get( G_OBJECT(f),"xmin",&xmin,NULL);
            if (xmin0==-1.)
                xmin0 = xmin;

            if (xmin==xmin0)
                g_object_set( G_OBJECT(f),"xmin",xmin1,NULL);
            else
                g_object_set( G_OBJECT(f),"xmin",xmin0,NULL);
        }
        if (gg)
            gtk_gauge_set_value(gg, randgauss(0.5*(gg->xmin+gg->xmax), (gg->xmax-gg->xmin)/sqrt(12.)));
    }

    return 1;
}
static gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
    /* If you return FALSE in the "delete_event" signal handler,
     * GTK will emit the "destroy" signal. Returning TRUE means
     * you don't want the window to be destroyed.
     * This is useful for popping up 'are you sure you want to quit?'
     * type dialogs. */

    g_print ("delete event occurred\n");

    /* Change TRUE to FALSE and the main window will be destroyed with
     * a "delete_event". */

    return TRUE;
}

int main(int argc, char *argv[])
{
    GtkWidget *mainWindow;
    SFunc *func;
    GList *lf;
    GObject *obj;
    gchar *func_name;

    gtk_set_locale ();
    gtk_init (&argc, &argv);

    std_logger = g_set_printerr_handler(my_logger );

    func = g_new(SFunc,1);
    func->ff = (HstFunction *)hst_histogram_new(51, -25.5, 25.5, 0);
    func->freq = 1.;
    func->mean = -10.;
    func->sigma= 2.;
    g_object_set(G_OBJECT(func->ff),"name","func_name",NULL);
    g_object_get(G_OBJECT(func->ff),"name",&func_name,NULL);
    printf("The function name is %s\n", func_name);
    g_free(func_name);
    hst_function_set_name(func->ff, "Function 1");
    LFunc = g_list_append(NULL,func);

    func = g_new(SFunc,1);
    func->ff = (HstFunction *)hst_histogram_new(51, -25.5, 25.5, 0);
    hst_function_set_name(func->ff, "Function 2");
    func->freq = 0.5;
    func->mean = 10.;
    func->sigma= 2.;
    LFunc = g_list_append(LFunc, func);

    func = g_new(SFunc,1);
    func->ff = (HstFunction *)hst_histogram_new(51, -25.5, 25.5, 0);
    hst_function_set_name(func->ff, "Function 3");
    func->freq = 0.75;
    func->mean = 25.;
    func->sigma= 2.;
    obj = G_OBJECT(func->ff);

    LFunc = g_list_append(LFunc, func);

    lf = g_list_first(LFunc);
    for (; lf;)
    {
        SFunc *sf = (SFunc *)lf->data;
        g_signal_connect(sf->ff,"range-changed",G_CALLBACK(range_changed),NULL);
        lf = g_list_next(lf);
    }

    /* try with this one the range-change signal */
    g_print("Changing nx\n");
    g_object_set(obj, "nx", 51, NULL);

    g_print("Changing nx and xmin\n");
    g_object_set(obj, "nx", 50, "xmin", -50., NULL);

    g_print("Changing nx,xmin and xmax\n");
    g_object_set(obj, "nx", 51, "xmin", -25.5, "xmax", 25.5, NULL);

    mainWindow = create_mainWindow ();
    gtk_widget_show (mainWindow);

    /* With this we test the range synchrnization */
    if (theHst)
        gtk_hst1d_sync_range(theHst, TRUE);

    gtk_idle_add ((GtkFunction)idle_func,mainWindow);
    gtk_quit_add(1, quitting_gtk, mainWindow);

    range_timer = g_timer_new();
    g_timer_start(range_timer);

    gtk_main ();

    return 0;
}

GtkWidget *create_mainWindow(void)
{
    GtkWidget *mainWindow;
    GtkWidget *vbox, *table;
    GtkWidget *hbuttonbox;
    GtkWidget *button1, *label;
    GtkWidget *hst1d, *wtracer, *wfunc;
    GtkWidget *frmHst, *frmTracer, *frmFunc, *frmHst2D;
    GtkWidget *hst2d;
    GtkWidget *frmGauge, *frmGraph, *frmProfile;
    GtkWidget *gauge;
    HstGraph *graph;
    HstProfile *hprof;
    HstFunction *func;
    double *xv, *yv, t;
    int ipt, npts=250;
    GtkWidget *textview;
    HstHistogram2D *hh;
    GList *lf;

    mainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (mainWindow), "tstHst");
    g_signal_connect (G_OBJECT (mainWindow), "delete_event",
                    G_CALLBACK (delete_event), NULL);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (mainWindow), vbox);

    table = gtk_table_new(4, 3, FALSE);
    gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);

    /*
     * Frame for the histogram
     */
    frmHst = gtk_frame_new (NULL);
    gtk_table_attach (GTK_TABLE (table), frmHst, 0, 1, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
    gtk_frame_set_shadow_type (GTK_FRAME (frmHst), GTK_SHADOW_IN);
    label = gtk_label_new ("A histogram");
    gtk_frame_set_label_widget (GTK_FRAME (frmHst), label);

    /* The histogram */
    lf = g_list_first(LFunc);
    hst1d = gtk_hst1d_new(((SFunc *)lf->data)->ff);
    theHst = GTK_HST1D(hst1d);
    gtk_hst1d_set_xtitle(GTK_HST1D(hst1d), "X axis");
    gtk_hst1d_set_ytitle(GTK_HST1D(hst1d), "Y axis");
    gtk_hst1d_filled(GTK_HST1D(hst1d), TRUE);
    for (lf=g_list_next (lf); lf; lf=g_list_next (lf))
    {
        SFunc *sf = (SFunc *)lf->data;
        gtk_hst1d_add_curve(GTK_HST1D(hst1d), sf->ff);
    }
    gtk_hst1d_set_curve_colors(theHst, "medium sea green", "royal blue", "firebrick", "gold", "turquoise", 
    NULL);

    gtk_hst1d_monitor_start(GTK_HST1D(hst1d), 500);
    gtk_container_add (GTK_CONTAINER (frmHst), hst1d);
    GTK_WIDGET_UNSET_FLAGS (hst1d, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (hst1d, GTK_CAN_DEFAULT);

    /*
     * frame for the  tracer
     */
    frmTracer = gtk_frame_new (NULL);
    gtk_table_attach (GTK_TABLE (table), frmTracer, 1, 2, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
    gtk_frame_set_shadow_type (GTK_FRAME (frmTracer), GTK_SHADOW_IN);
    label = gtk_label_new ("A tracer");
    gtk_frame_set_label_widget (GTK_FRAME (frmTracer), label);

    tracer = hst_tracer_new(51);
    wtracer = gtk_hst1d_new((HstFunction *)tracer);
    gtk_hst1d_set_colors(GTK_HST1D(wtracer), "foreground", "red", "background", "black", "axis", "green", NULL);
    gtk_hst1d_draw_y_axis(GTK_HST1D(wtracer), HST_AXIS_MIDDLE);
    gtk_hst1d_monitor_start(GTK_HST1D(wtracer), 150);
    gtk_container_add (GTK_CONTAINER (frmTracer), wtracer);
    GTK_WIDGET_UNSET_FLAGS (wtracer, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (wtracer, GTK_CAN_DEFAULT);

    /*
     * frame for a function
     */
    frmFunc = gtk_frame_new (NULL);
    gtk_table_attach (GTK_TABLE (table), frmFunc, 2, 3, 0, 1,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
    gtk_frame_set_shadow_type (GTK_FRAME (frmFunc), GTK_SHADOW_IN);
    label = gtk_label_new ("A function");
    gtk_frame_set_label_widget (GTK_FRAME (frmFunc), label);

    func = hst_function_new(51, -5., 5., erf_plus_one);
    hst_function_set_name(func, "erf(x+2)");
    wfunc = gtk_hst1d_new(func );
    func = hst_function_new(51, -5., 5., tanh_plus_one);
    hst_function_set_name(func, "tanh(x-2)");
    gtk_hst1d_add_curve(GTK_HST1D(wfunc), func);
    gtk_hst1d_set_curve_colors(GTK_HST1D(wfunc), "medium sea green", "royal blue", "firebrick", "gold", "turquoise", 
    NULL);

    gtk_hst1d_set_xtitle(GTK_HST1D(wfunc), "X axis");
    gtk_hst1d_set_ytitle(GTK_HST1D(wfunc), "Y axis");
    gtk_container_add (GTK_CONTAINER (frmFunc), wfunc);
    GTK_WIDGET_UNSET_FLAGS (wfunc, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (wfunc, GTK_CAN_DEFAULT);

    /*
     * A 2D histogram
     */
    frmHst2D = gtk_frame_new (NULL);
    gtk_table_attach (GTK_TABLE (table), frmHst2D, 0, 1, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
    gtk_frame_set_shadow_type (GTK_FRAME (frmHst2D), GTK_SHADOW_IN);
    label = gtk_label_new ("A 2D histogram");
    gtk_frame_set_label_widget (GTK_FRAME (frmHst2D), label);

    hh = hst_histogram2d_new (101, -25., 25., 101, -25., 25., 0);
    hst2d = gtk_hst2d_new ( HST_HISTOGRAM2D(hh));
    gtk_hst2d_set_squared(GTK_HST2D(hst2d),TRUE);
    g_object_set_data( G_OBJECT(mainWindow),"h2d",hh);
    gtk_hst1d_monitor_start(GTK_HST1D(hst2d), 500);
    gtk_container_add (GTK_CONTAINER (frmHst2D), hst2d);
    GTK_WIDGET_UNSET_FLAGS (hst2d, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (hst2d, GTK_CAN_DEFAULT);

    /*
     * The graph
     */
    frmGraph = gtk_frame_new (NULL);
    gtk_table_attach (GTK_TABLE (table), frmGraph, 1, 2, 1, 2,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
    gtk_frame_set_shadow_type (GTK_FRAME (frmGraph), GTK_SHADOW_IN);
    label = gtk_label_new ("A Graph");
    gtk_frame_set_label_widget (GTK_FRAME (frmGraph), label);

    xv = g_new(double, npts);
    yv = g_new(double, npts);
    for (ipt=0, t=0.; ipt<npts; ipt++, t+=0.1)
    {
        double r = exp(0.1*t);
        xv[ipt] = r * cos(t);
        yv[ipt] = r * sin(t);
    }
    graph = hst_graph_new(npts, xv, yv);
    hst1d = gtk_hst1d_new( HST_FUNCTION(graph));
    gtk_hst1d_set_xtitle(GTK_HST1D(hst1d), "X axis");
    gtk_hst1d_set_ytitle(GTK_HST1D(hst1d), "Y axis");
    gtk_container_add (GTK_CONTAINER (frmGraph), hst1d);
    g_free(xv);
    g_free(yv);

    /*
     * The gauge<
     */
    frmGauge = gtk_frame_new (NULL);
    gtk_table_attach (GTK_TABLE (table), frmGauge, 2, 3, 1, 2,
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL), 0, 0);
    gtk_frame_set_shadow_type (GTK_FRAME (frmGauge), GTK_SHADOW_IN);
    label = gtk_label_new ("A Gauge");
    gtk_frame_set_label_widget (GTK_FRAME (frmGauge), label);

    gauge = gtk_gauge_new(0., 50.e3);
    //    gtk_gauge_set_squared(GTK_GAUGE(gauge),TRUE);
    gtk_gauge_set_value(GTK_GAUGE(gauge),10.);
    gtk_gauge_set_title(GTK_GAUGE(gauge),"gauge");
    gtk_container_add (GTK_CONTAINER (frmGauge), gauge);
    GTK_WIDGET_UNSET_FLAGS (gauge, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (gauge, GTK_CAN_DEFAULT);
    g_object_set_data( G_OBJECT(mainWindow),"gauge",gauge);

    /* Profile */
    label = gtk_label_new("X");
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL), 0, 0);

    frmProfile = gtk_frame_new ("Profile");
    gtk_table_attach (GTK_TABLE (table), frmProfile, 1, 2, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL), 0, 0);
    gtk_frame_set_shadow_type (GTK_FRAME (frmProfile), GTK_SHADOW_IN);

    hprof = hst_profile_new(50, 0., 10., NULL);
    hst1d = gtk_hst1d_new( HST_FUNCTION(hprof));
    gtk_hst1d_monitor_start(GTK_HST1D(hst1d), 500);
    gtk_container_add (GTK_CONTAINER (frmProfile), hst1d);
    g_object_set_data( G_OBJECT(mainWindow),"hprof",hprof);

    label = gtk_label_new("X");
    gtk_table_attach (GTK_TABLE (table), label, 2, 3, 2, 3,
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL), 0, 0);

    /* Doc */
    textview = gtk_text_view_new ();
    gtk_table_attach (GTK_TABLE (table), textview, 0, 3, 3, 4,
    (GtkAttachOptions) (GTK_FILL),
    (GtkAttachOptions) (GTK_SHRINK),
    0, 0);
    gtk_container_set_border_width (GTK_CONTAINER (textview), 5);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
    gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview), GTK_WRAP_WORD);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), FALSE);
    gtk_text_view_set_left_margin (GTK_TEXT_VIEW (textview), 5);
    gtk_text_view_set_right_margin (GTK_TEXT_VIEW (textview), 5);
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview)), "You can choose a given range in any of the axis of a 1D histogram"
        "by selecting it with the left button of the mouse "
        "while preessing\n"
        "\tCTRL  for the X axis or\n"
        "\tSHIFT for the Y axis\n"
        "For a 2D histogram you can choose the Z range on the color scale "
        "whith the left button of the mouse.\n"
        "The original range is restored presing "
        "the mouse middle button.\n"
        "Click the mouse right button to explore the popup menu "
        "and change some of the histogram attributes.", -1);

    hbuttonbox = gtk_hbutton_box_new ();
    gtk_box_pack_start (GTK_BOX (vbox), hbuttonbox, FALSE, TRUE, 0);

    button1 = gtk_button_new_from_stock ("gtk-ok");
    gtk_container_add (GTK_CONTAINER (hbuttonbox), button1);
    GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

    g_signal_connect ((gpointer) button1, "clicked",
                    G_CALLBACK (gtk_main_quit),
                    NULL);

    gtk_widget_show_all(mainWindow);
    return mainWindow;
}
