#include <stdio.h>
#include <math.h>
#include <gtk-hst/function.h>
#include <gtk-hst/function2d.h>
#include <gtk-hst/histogram.h>
#include <gtk-hst/histogram2d.h>
#include <gtk-hst/tracer.h>
#include <random.h>

void test_func(HstFunction *ff) {
    gboolean own=FALSE;
    if (ff==NULL) {
	own=TRUE;
	ff = hst_function_new(20,0.,20.,sin);
    }

    printf("\nTesting HstFunction: sin(PI/2)=%f\n",
	   hst_function_eval(ff,M_PI_2)
	);
    
    if (own)
	g_object_unref(ff);
}

void test_hst(const char *txt, HstHistogram *hst, double mean, double stdv) {
    gboolean own=FALSE;
    int i;
    
    if (hst==NULL) {
	own=TRUE;
	hst = hst_histogram_new(51,-25.5,25.5,0);
    }
    printf("\nTesting %s class: <x>=%.2f +/ %.2f\n",
	   txt,mean,stdv);

    for (i=0;i<50000;i++) {
	double v = randgauss(mean,stdv);
	hst_histogram_fill(hst,v,1.);
    }
    printf("N. entries: %f\n",hst_histogram_n(hst));
    printf("Mean      : %f\n",hst_histogram_mean(hst));
    printf("stdev     : %f\n",hst_histogram_stdev(hst));
    printf("Value range: [%f,%f]\n",hst_histogram_xmin(hst),hst_histogram_xmax(hst));
    
    
    if (own)
	g_object_unref(hst);
}


void test_hst2d(HstHistogram2D *h2d,
		double mean1, double stdv1,
		double mean2, double stdv2,
		double corr)
{
    int i;
    double f1,f2;
    gboolean own=FALSE;
    HstStats *S;
    if (h2d==NULL) {
	own = TRUE;
	h2d = hst_histogram2d_new(50,mean1-5.*stdv1,mean2+5.*stdv1,
				  50,mean2-5.*stdv2,mean2+5.*stdv2,
				  0);
    }

    f1 = sqrt((1.+corr)/2.);
    f2 = sqrt((1.-corr)/2.);

    printf("\nStats <x>=%.2f +/- %.3f <y>=%.2f +/- %.3f corr=%.2f\n",
	   mean1,stdv1,mean2,stdv2,corr);
    

    for (i=0;i<50000;i++) {
	double x = randgauss( 0.,1.);
	double y = randgauss( 0.,1.);
	
	hst_histogram2d_fill(h2d, mean1 + stdv1*(f1*x+f2*y), mean2 + stdv2*(f1*x-f2*y),1.);
    }

    S = hst_stats_new();
    hst_histogram2d_get_stats(h2d,S);
    printf("N. entries   : %f\n",hst_stats_get_nc(S));
    printf("X Mean       : %f\n",hst_stats_get_mean(S,0));
    printf("X stdev      : %f\n",hst_stats_get_stdev(S,0));
    printf("Y Mean       : %f\n",hst_stats_get_mean(S,1));
    printf("Y stdev      : %f\n",hst_stats_get_stdev(S,1));
    printf("XY corr.     : %f\n",hst_stats_get_correlation(S));
    printf("X value range: [%f,%f]\n",hst_stats_get_min(S,0),hst_stats_get_max(S,0));
    printf("Y value range: [%f,%f]\n",hst_stats_get_min(S,1),hst_stats_get_max(S,0));

    g_object_unref(S);
    if (own)
	g_object_unref(h2d);
}

double f2d(double x, double y) {
    return x+y;
}
void test_func2d(HstFunction2D *ff,double x, double y) {
    gboolean own=FALSE;
    int npts,nx,ny;
    double xmin,xmax,ymin,ymax,xstep,ystep;
    GObject *o;
    if (ff==NULL) {
	own=TRUE;
	ff = hst_function2d_new(20,0.,10.,40,0.,40.,f2d);
    }
    
    printf("\nTesting HstFunction2D: f(%f,%f)=x+y=%f\n",
	   x,y,hst_function2d_eval(ff,x,y)
	);

    o = G_OBJECT(ff);


    g_object_get( o,"npts",&npts,"nx",&nx,"xmin",&xmin,"xmax",&xmax,"xstep", &xstep,"ny",&ny,"ymin",&ymin,"ymax",&ymax,"ystep", &ystep,NULL);
    printf("npts: %d\n",npts);
    printf("nx %d xmin %f xmax %f xstep %f\n",nx,xmin,xmax,xstep);
    printf("ny %d ymin %f ymax %f ystep %f\n",ny,ymin,ymax,ystep);
    
    if (own)
	g_object_unref(ff);

}

int main() {
    
    HstTracer *T;

    /* Initialize the GObject interface */
    g_type_init();

    /* test the HstFunction object */
    test_func(NULL);

    /* Test the HstHistogram class */
    test_hst("Histogram",NULL,0.,2.);

    /* Test the HstTracer class */
    T = hst_tracer_new( 50 );    
    test_hst("Tracer",HST_HISTOGRAM(T),1.,1.);
    g_object_unref(T);

    /* Test the HstFunction2D object */
    test_func2d(NULL,1.,2);

    /* Test the stats */
    test_hst2d(NULL,1.,2.,2.,1.,0.75);

    return 0;
}
